# Makefile for Quake III Arena SDK

QVM=		yes

# Compiler flags

.ifdef TEAMARENA
TA_FLAGS=	-DMISSIONPACK
.endif

INCLUDES=	-I${CGAME_DIR} -I${GAME_DIR} -I${UI_DIR}
CFLAGS+=	${INCLUDES} ${TA_FLAGS}

# Tools

Q3ASM=		q3asm
Q3ASM_FLAGS=	

Q3LCC=		q3lcc
Q3LCC_FLAGS=	${INCLUDES} ${TA_FLAGS} -DQ3_VM -S -Wf-target=bytecode -Wf-g

# Building information

BUILD_DIR=	build

CGAME_NAME=	cgame
GAME_NAME=	game
UI_NAME=	ui

QVM_SUFX=	.qvm
SO_SUFX=	.so

.for f in CGAME GAME UI
${f}_QVM=	${BUILD_DIR}/${${f}_NAME}${QVM_SUFX}
${f}_SO=	${BUILD_DIR}/${${f}_NAME}${SO_SUFX}
.endfor

CGAME_DIR=	code/cgame
GAME_DIR=	code/game
Q3_UI_DIR=	code/q3_ui
TA_UI_DIR=	code/ui
.ifdef TEAMARENA
UI_DIR=		${TA_UI_DIR}
.else
UI_DIR=		${Q3_UI_DIR}
.endif

CGAME_SRCS=	${CGAME_DIR}/cg_main.c \
		${GAME_DIR}/bg_misc.c \
		${GAME_DIR}/bg_pmove.c \
		${GAME_DIR}/bg_slidemove.c \
		${GAME_DIR}/q_math.c \
		${GAME_DIR}/q_shared.c \
		${CGAME_DIR}/cg_consolecmds.c   \
		${CGAME_DIR}/cg_draw.c \
		${CGAME_DIR}/cg_drawtools.c \
		${CGAME_DIR}/cg_effects.c \
		${CGAME_DIR}/cg_ents.c \
		${CGAME_DIR}/cg_event.c \
		${CGAME_DIR}/cg_info.c \
		${CGAME_DIR}/cg_localents.c \
		${CGAME_DIR}/cg_marks.c \
		${CGAME_DIR}/cg_players.c \
		${CGAME_DIR}/cg_playerstate.c \
		${CGAME_DIR}/cg_predict.c \
		${CGAME_DIR}/cg_scoreboard.c \
		${CGAME_DIR}/cg_servercmds.c \
		${CGAME_DIR}/cg_snapshot.c \
		${CGAME_DIR}/cg_view.c \
		${CGAME_DIR}/cg_weapons.c

CGAME_SRCS_QVM=	${CGAME_SRCS} \
		${GAME_DIR}/bg_lib.c \
		${CGAME_DIR}/cg_syscalls.asm

CGAME_SRCS_SO=	${CGAME_SRCS} \
		${CGAME_DIR}/cg_syscalls.c

GAME_SRCS=	${GAME_DIR}/g_main.c \
		${GAME_DIR}/ai_chat.c \
		${GAME_DIR}/ai_cmd.c \
		${GAME_DIR}/ai_dmnet.c \
		${GAME_DIR}/ai_dmq3.c \
		${GAME_DIR}/ai_main.c \
		${GAME_DIR}/ai_team.c \
		${GAME_DIR}/ai_vcmd.c \
		${GAME_DIR}/bg_misc.c \
		${GAME_DIR}/bg_pmove.c \
		${GAME_DIR}/bg_slidemove.c \
		${GAME_DIR}/g_active.c \
		${GAME_DIR}/g_arenas.c \
		${GAME_DIR}/g_bot.c \
		${GAME_DIR}/g_client.c \
		${GAME_DIR}/g_cmds.c \
		${GAME_DIR}/g_combat.c \
		${GAME_DIR}/g_items.c \
		${GAME_DIR}/g_mem.c \
		${GAME_DIR}/g_misc.c \
		${GAME_DIR}/g_missile.c \
		${GAME_DIR}/g_mover.c \
		${GAME_DIR}/g_session.c \
		${GAME_DIR}/g_spawn.c \
		${GAME_DIR}/g_svcmds.c \
		${GAME_DIR}/g_target.c \
		${GAME_DIR}/g_team.c \
		${GAME_DIR}/g_trigger.c \
		${GAME_DIR}/g_utils.c \
		${GAME_DIR}/g_weapon.c \
		${GAME_DIR}/q_math.c \
		${GAME_DIR}/q_shared.c

GAME_SRCS_QVM=	${GAME_SRCS} \
		${GAME_DIR}/bg_lib.c \
		${GAME_DIR}/g_syscalls.asm

GAME_SRCS_SO=	${GAME_SRCS} \
		${GAME_DIR}/g_syscalls.c

.ifdef TEAMARENA
UI_SRCS=	${UI_DIR}/ui_main.c \
		${UI_DIR}/ui_atoms.c \
		${UI_DIR}/ui_gameinfo.c \
		${UI_DIR}/ui_players.c \
		${UI_DIR}/ui_util.c \
		${UI_DIR}/ui_shared.c \
		${GAME_DIR}/bg_misc.c \
		${GAME_DIR}/q_math.c \
		${GAME_DIR}/q_shared.c
.else
UI_SRCS=	${UI_DIR}/ui_main.c \
		${GAME_DIR}/bg_misc.c \
		${GAME_DIR}/q_math.c \
		${GAME_DIR}/q_shared.c \
		${UI_DIR}/ui_addbots.c \
		${UI_DIR}/ui_atoms.c \
		${UI_DIR}/ui_cdkey.c \
		${UI_DIR}/ui_cinematics.c \
		${UI_DIR}/ui_confirm.c \
		${UI_DIR}/ui_connect.c \
		${UI_DIR}/ui_controls2.c \
		${UI_DIR}/ui_credits.c \
		${UI_DIR}/ui_demo2.c \
		${UI_DIR}/ui_display.c \
		${UI_DIR}/ui_gameinfo.c \
		${UI_DIR}/ui_ingame.c \
		${UI_DIR}/ui_menu.c \
		${UI_DIR}/ui_mfield.c \
		${UI_DIR}/ui_mods.c \
		${UI_DIR}/ui_network.c \
		${UI_DIR}/ui_options.c \
		${UI_DIR}/ui_playermodel.c \
		${UI_DIR}/ui_players.c \
		${UI_DIR}/ui_playersettings.c \
		${UI_DIR}/ui_preferences.c \
		${UI_DIR}/ui_qmenu.c \
		${UI_DIR}/ui_removebots.c \
		${UI_DIR}/ui_serverinfo.c \
		${UI_DIR}/ui_servers2.c \
		${UI_DIR}/ui_setup.c \
		${UI_DIR}/ui_sound.c \
		${UI_DIR}/ui_sparena.c \
		${UI_DIR}/ui_specifyserver.c \
		${UI_DIR}/ui_splevel.c \
		${UI_DIR}/ui_sppostgame.c \
		${UI_DIR}/ui_spskill.c \
		${UI_DIR}/ui_startserver.c \
		${UI_DIR}/ui_team.c \
		${UI_DIR}/ui_teamorders.c \
		${UI_DIR}/ui_video.c
.endif

UI_SRCS_QVM=	${UI_SRCS} \
		${GAME_DIR}/bg_lib.c \
		${TA_UI_DIR}/ui_syscalls.asm

UI_SRCS_SO=	${UI_SRCS} \
		${TA_UI_DIR}/ui_syscalls.c

# Rules

all: help

help:
	@${ECHO}
	@${ECHO} "============================================================"
	@${ECHO} "Available targets:"
	@${ECHO}
	@${ECHO} "help	-	Display this help message."
	@${ECHO} "qvm	-	Build portable QVM (Quake Virtual Machine)."
	@${ECHO} "so	-	Build platform-specific SO (shared objects)."
	@${ECHO} "clean	-	Clean the build directory."
	@${ECHO} "============================================================"
	@${ECHO}

# Rules for building QVM

qvm: ${CGAME_QVM} ${GAME_QVM} ${UI_QVM}

.for f in CGAME GAME UI
${${f}_QVM}: ${${f}_SRCS_QVM:C/.asm$/.q3asm/:C/.c$/.q3asm/}
	${Q3ASM} -o $@ ${.ALLSRC:T:C/^/${BUILD_DIR}\//}
.endfor

.if make(qvm)

.SUFFIXES :
.SUFFIXES : .q3asm .asm .c
.asm.q3asm :
	cp $< ${BUILD_DIR}/${.TARGET:T}
.c.q3asm  :
	${Q3LCC} ${Q3LCC_FLAGS} -o ${BUILD_DIR}/${.TARGET:T} $<
.endif

# Rules for building SO

so: ${CGAME_SO} ${GAME_SO} ${UI_SO}

.for f in CGAME GAME UI
${${f}_SO}: ${${f}_SRCS_SO:C/.c$/.o/}
	${CC} ${CFLAGS} -shared -o ${BUILD_DIR}/${.TARGET:T} ${.ALLSRC:T:C/^/${BUILD_DIR}\//}
.endfor

.if make(so)

.SUFFIXES :
.SUFFIXES : .o .c
.c.o      :
	${CC} ${CFLAGS} -c -fPIC -o ${BUILD_DIR}/${.TARGET:T} $<
.endif

clean:
	rm -f depend
	(cd ${CGAME_DIR} && rm -f *.o *.q3asm *.qvm)
	(cd ${GAME_DIR} && rm -f *.o *.q3asm *.qvm)
	(cd ${UI_DIR} && rm -f *.o *.q3asm *.qvm)
